

-- Set to false to use file-scoped variables or true to use the new addon-scoped variables
local useAddonScope = true
local addonName, MenuClass

if useAddonScope then
    addonName, MenuClass = ...
else
    addonName, MenuClass = "Global TMog", {}
end

function MenuClass:New()
    local ret = {}
    
    -- set the defaults
    ret.menuList = {}
    ret.anchor = 'cursor'; -- default at the cursor
    ret.x = nil;
    ret.y = nil;
    ret.displayMode = 'MENU'; -- default
    ret.autoHideDelay = nil;
    ret.menuFrame = nil; -- If not defined, :Show() will create a generic menu frame
    ret.uniqueID = 1

    -- import the functions
    for k,v in pairs(self) do
        ret[k] = v
    end
    
    -- return a copy of the class
    return ret
end

--[[
    Return the index where "text" lives.
    ; text : The text to search for.
--]]
function MenuClass:GetItemByText(text)
    for k,v in pairs(self.menuList) do
        if v.text == text then
            return k
        end
    end
end

--[[
    Add menu items
    ; text : The display text.
    ; func : The function to execute OnClick.
    ; isTitle : 1 if this is a header (usually the first one)
    ; otherAttributes : table - { ["attribute"] = value, }
    returns the last index of the menu item that was just added.
--]]
function MenuClass:AddItem(text, func, isTitle, otherAttributes)
    local info = {}
    
    info["text"] = text
    info["isTitle"] = isTitle
    info["func"] = func
    
    if type(otherAttributes) == "table" then
        for attribute, value in pairs(otherAttributes) do
            info[attribute] = value
        end
    end

    table.insert(self.menuList, info)
    return #self.menuList
end

--[[
    Set an attribute for the menu item.
    Valid attributes are found in the FrameXML\UIDropDownMenu.lua file with their valid values.
    Arbitrary non-official attributes are allowed, but are only useful if you plan to access them with :GetAttribute().
    ; text : The text of the menu item or index of the menu item.
    ; attribute : Set this attribute to "value".
    ; value : The value to set the attribute to.
--]]
function MenuClass:SetAttribute(text, attribute, value)
    self.menuList[self:GetItemByText(text) or (self.menuList[text] and text) or 1][attribute or "uniqueID"] = value
end

--[[
    Get an attribute for the menu item.
    Valid attributes are found in the FrameXML\UIDropDownMenu.lua file with their valid values or any arbitrary attribute set with :SetAttribute().
    ; text : The text of the menu item or index of the menu item.
    ; attribute : Get this attribute.
--]]
function MenuClass:GetAttribute(text, attribute)
    return self.menuList[self:GetItemByText(text) or (self.menuList[text] and text) or 1][attribute or "uniqueID"]
end

--[[
    Remove the first item matching "text"
    ; text : The text to search for.
--]]
function MenuClass:RemoveItem(text)
    if self:GetItemByText(text) == nil then
    else
        table.remove(self.menuList, self:GetItemByText(text))
    end
end

function MenuClass:ItemExists(text)
    if self:GetItemByText(text) == nil then
        return false
    else
        return true
    end
end

--[[
    ; anchor : Set the anchor point. 
--]]
function MenuClass:SetAnchor(anchor)
    if anchor ~= 'cursor' then
        self.x = 0
        self.y = 0
    end
    self.anchor = anchor
end

--[[
    ; displayMode : "MENU"
--]]
function MenuClass:SetDisplayMode(displayMode)
    self.displayMode = displayMode
end

--[[
    ; autoHideDelay : How long, without a click, before the menu goes away.
--]]
function MenuClass:SetAutoHideDelay(autoHideDelay)
    self.autoHideDelay = tonumber(autoHideDelay)
end

--[[
    ; menuFrame : Should inherit a Drop Down Menu template.
--]]
function MenuClass:SetMenuFrame(menuFrame)
    self.menuFrame = menuFrame
end

function MenuClass:GetMenuList()
    return self.menuList
end

--[[
    ; x : X position
    ; save : When not nil, will add to the current value rather than replace it
--]]
function MenuClass:SetX(x, save)
    if save then
        self.x = self.x + x
    else
        self.x = x
    end
end

--[[
    ; y : Y position
    ; save : When not nil, will add to the current value rather than replace it
--]]
function MenuClass:SetY(y, save)
    if save then
        self.y = self.y + y
    else
        self.y = y
    end
end

function MenuClass:Activate()
    if not self.menuFrame then
        while _G['GenericMenuClassFrame'..self.uniqueID] do -- ensure that there's no namespace collisions
            self.uniqueID = self.uniqueID + 1
        end
        -- the frame must be named for some reason
        self.menuFrame = CreateFrame('Frame', 'GenericMenuClassFrame'..self.uniqueID, UIParent, "UIDropDownMenuTemplate")
    end
    self.menuFrame.menuList = self.menuList
end

--[[
    Show the menu.
--]]
function MenuClass:Show()
    self:Activate()
    EasyMenu(self.menuList, self.menuFrame, self.anchor, self.x, self.y, self.displayMode, self.autoHideDelay)
end

-- If you're not using the addon-scoped variables, you must have a global variable in order to use this menu.
if not useAddonScope then
    _G[addonName.."Menu"] = MenuClass
end


local menu = MenuClass:New()

local loadButton
local saveButton
local textBox
local deleteButton

function createGTMButtons()
    if loadButton == nil then 
        loadButton = CreateFrame("Button", "StupidButton", WardrobeFrame)
        loadButton:SetPoint("TOPLEFT")
        loadButton:SetWidth(75)
        loadButton:SetHeight(25)
        
        loadButton:SetText("Load")
        loadButton:SetNormalFontObject("GameFontNormal")
        
        local ntex = loadButton:CreateTexture()
        ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
        ntex:SetTexCoord(0, 0.625, 0, 0.6875)
        ntex:SetAllPoints()	
        loadButton:SetNormalTexture(ntex)
        
        local htex = loadButton:CreateTexture()
        htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
        htex:SetTexCoord(0, 0.625, 0, 0.6875)
        htex:SetAllPoints()
        loadButton:SetHighlightTexture(htex)
        
        local ptex = loadButton:CreateTexture()
        ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
        ptex:SetTexCoord(0, 0.625, 0, 0.6875)
        ptex:SetAllPoints()
        loadButton:SetPushedTexture(ptex)
        
        
        loadButton:SetScript('OnClick', function(self, button)
            menu:Show()
        end)
    end
    if saveButton == nil then 
        saveButton = CreateFrame("Button", "DumbButton", WardrobeFrame)
        saveButton:SetPoint("TOPLEFT",80,0)
        saveButton:SetWidth(75)
        saveButton:SetHeight(25)
        
        saveButton:SetText("Save")
        saveButton:SetNormalFontObject("GameFontNormal")
        
        local ntex = saveButton:CreateTexture()
        ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
        ntex:SetTexCoord(0, 0.625, 0, 0.6875)
        ntex:SetAllPoints()	
        saveButton:SetNormalTexture(ntex)
        
        local htex = saveButton:CreateTexture()
        htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
        htex:SetTexCoord(0, 0.625, 0, 0.6875)
        htex:SetAllPoints()
        saveButton:SetHighlightTexture(htex)
        
        local ptex = saveButton:CreateTexture()
        ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
        ptex:SetTexCoord(0, 0.625, 0, 0.6875)
        ptex:SetAllPoints()
        saveButton:SetPushedTexture(ptex)


        saveButton:SetScript('OnClick', createProfile)
    end
    if editBox == nil then
        editBox = CreateFrame("EditBox", "StupidBox", WardrobeFrame,"InputBoxTemplate")
        editBox:SetPoint("TOPLEFT", 165,0)
        editBox:SetWidth(75)
        editBox:SetHeight(25)
        editBox:SetAutoFocus(false);
        editBox:SetText("")
    end

    if deleteButton == nil then 
        deleteButton = CreateFrame("Button", "StupidButton", WardrobeFrame)
        deleteButton:SetPoint("TOPLEFT",250,0)
        deleteButton:SetWidth(50)
        deleteButton:SetHeight(25)
        
        deleteButton:SetText("Delete")
        deleteButton:SetNormalFontObject("GameFontNormal")
        
        local ntex = deleteButton:CreateTexture()
        ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
        ntex:SetTexCoord(0, 0.625, 0, 0.6875)
        ntex:SetAllPoints()	
        deleteButton:SetNormalTexture(ntex)
        
        local htex = deleteButton:CreateTexture()
        htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
        htex:SetTexCoord(0, 0.625, 0, 0.6875)
        htex:SetAllPoints()
        deleteButton:SetHighlightTexture(htex)
        
        local ptex = deleteButton:CreateTexture()
        ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
        ptex:SetTexCoord(0, 0.625, 0, 0.6875)
        ptex:SetAllPoints()
        deleteButton:SetPushedTexture(ptex)
        
        
        deleteButton:SetScript('OnClick', deleteOutfit)
    end
end

profile = {}

local profiles = {
    cloth = {},
    leather = {},
    mail = {},
    plate = {},
}


local _,_,armorType = UnitClass("player")


function setArmorType()
    if armorType == 10 or armorType == 11 or armorType == 12 or armorType == 4 then
        armorType = profiles.leather
    elseif armorType == 1 or armorType == 2 or armorType == 6 then
        armorType = profiles.plate
    elseif armorType == 5 or armorType == 8 or armorType == 9 then
        armorType = profiles.cloth
    elseif armorType == 3 or armorType == 7 then
        armorType = profiles.mail

    end
end


function deleteOutfit()
    local text = editBox:GetText()
    if text == "" then

    else
        menu:RemoveItem(text)
        armorType[text] = nil
        editBox:SetText("")
    end
end


function createProfile()

    local set = editBox:GetText()
    editBox:SetText("")

    if set == "" then
        print("|cffADFF2FGlobal TMog|r: Please enter a name for your new outfit.")
    else
        armorType[set] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
        for i = 1, 19, 1 do
            if i == 2 or i == 11 or i == 12 or i == 13 or i == 14 or i == 18 then
                armorType[set][i] = -1
            else
                local og,_,xmogd,_,id = C_Transmog.GetSlotVisualInfo(i,0)
                if id > 0 then
                    armorType[set][i] = id
                elseif xmogd > 0 then
                    armorType[set][i] = xmogd
                else
                    armorType[set][i] = og
                end
            end
        end

    if menu:ItemExists(set) == false then
        menu:RemoveItem('Close')
        menu:AddItem(set, function()
            LoadGTMAppearance(set)
        end)
        menu:AddItem('Close', function()
            -- do nothing, just close.
        end)
        end
    end
end

function LoadGTMAppearance(name)
    editBox:SetText(name)
   for i=1,19,1 do
        if armorType[i] ~= -1 then
            if i == 2 or i == 11 or i == 12 or i == 13 or i == 14 or i == 18 then
                --do nothing
            else
                C_Transmog.SetPending(i,0,armorType[name][i])
            end
           
        end
   end
end

--[[
    Adds all outfits saved using the default UI to GTM
]]
function PopulateWithOfficialOutfits()

    for i=0,C_TransmogCollection.GetNumMaxOutfits(),1 do
        local outfit = C_TransmogCollection.GetOutfitSources(i)
        local outfitName = C_TransmogCollection.GetOutfitName(i)
        --print(outfitName)
        if outfit ~= nil then

            if armorType[outfitName] == nil then
                armorType[outfitName] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
                for x=1,19,1 do
                    if x == 2 or x == 11 or x == 12 or x == 13 or x == 14 or x == 18 then
                        armorType[outfitName][x] = -1
                    else
                        armorType[outfitName][x] = outfit[x]
                    end
                end
            end

            menu:AddItem(outfitName, function()
                LoadGTMAppearance(outfitName)
            end)
        end
    end

    menu:AddItem('Close', function()
        -- do nothing, just close.
    end)
end

local function eventHandler(self, event, ...)
    
    if event == "TRANSMOGRIFY_OPEN" then
        createGTMButtons()
        
        return
    end

    if event == "ADDON_LOADED" then
        if (... == "Global TMog") then
            if GlobalTMogSettings ~= nil then
                profiles = GlobalTMogSettings
                setArmorType()
                for i, v in pairs(armorType) do
                    menu:AddItem(i, function()
                        LoadGTMAppearance(i)
                    end)
                end
            else
                GlobalTMogSettings = profiles
            end
            PopulateWithOfficialOutfits()
        end

        return
    end

    if event == "PLAYER_LOGOUT" then
        GlobalTMogSettings = profiles
        return
    end
end




local f = CreateFrame("Frame")
f:RegisterEvent("TRANSMOGRIFY_OPEN")
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGOUT")
f:SetScript("OnEvent", eventHandler)


